#!/usr/bin/perl

# Sample program for the ChronicSingleton sample.
# Copyright 2004 by Rocco Caputo.  Free software.
# Same terms as Perl itself.  Have fun!

use warnings;
use strict;

use POE;
use ChronicSingleton;

# Create a session to request and handle
# recurring events.

POE::Session->create(
  inline_states => {
    _start   => \&setup_session,
    got_tick => \&handle_tick,
  },
);

# Set up the session.  Request a recurring
# timer.

sub setup_session {
  $_[HEAP]->{count} = 0;
  $_[HEAP]->{ticker_id} =
    ChronicSingleton->start(
      Event => "got_tick",
      Interval => 1,
    );
}

# Receive and handle a recurring event.  Do some
# work, and eventually stop when we're done.

sub handle_tick {
  my ($kernel, $heap) = @_[KERNEL, HEAP];
  print "Poit #", ++$heap->{count}, "\n";
  ChronicSingleton->stop(
    Id => $heap->{ticker_id}
  ) if $heap->{count} >= 5;
}

# Run POE's main loop, otherwise nothing would
# happen.

POE::Kernel->run();
exit;
